@@ -4,7 +4,8 @@
 #' function. 
 #' 
 #' @details This function can accept either a \code{rollcall} data object from package
-#' \code{pscl} or a long data frame where one row equals one item-person (bill-legislator)
+#' \code{pscl} or 
+#' a long data frame where one row equals one item-person (bill-legislator)
 #' observation with associated outcome. The preferred method is the long data frame 
 #' as passing a long data frame permits
 #' the inclusion of a wide range of covariates in the model, such as person-varying and item-varying 
@@ -33,9 +34,13 @@
 #' data as ignorable missing data that will be removed (list-wise deletion) before estimating a
 #' model.
 #' 
+#' @section Time-Varying Models
+#' 
 #' To run a time-varying model, you need to include the name of a column with dates (or integers) that is passed 
 #' to the \code{time_id} option.
 #' 
+#' @section Continuous Outcomes
+#' 
 #' If the outcome is unbounded i.e. a continuous or an unbounded 
 #' discrete variable like Poisson, simply set \code{unbounded} to \code{TRUE}. You can ignore the
 #' options that specify which values should be \code{high_val} or \code{low_val}. You can either specify
@@ -43,6 +48,19 @@
 #' missing values (\code{NA}) will be recoded to a specific value out of the range of the outcome to use
 #' for modeling the missingness.
 #' 
+#' @section Hierarchical Covariates
+#' 
+#' Covariates can be fit on the person-level ideal point parameters as well as
+#' item discrimination parameters for either the inflated (missing) or non-inflated (observed) 
+#' models. These covariates must be columns that were included with the data fed to the 
+#' \code{\link{id_make}} function. The covariate relationships are specified as 
+#' one-sided formulas, i.e. \code{~cov1 + cov2 + cov1*cov2}. To interact covariates with the 
+#' person-level ideal points you can use \code{~cov1 + person_id + cov1*person_id} and for
+#' group-level ideal poins you can use \code{~cov1 + group_id + cov1*group_id} where
+#' \code{group_id} or \code{person_id} is the same name as the name of the column 
+#' for these options that you passed to \code{id_make} (i.e., the names of the columns
+#' in the original data).
+#' 
 #' @param score_data A data frame in long form, i.e., one row in the data for each 
 #' measured score or vote in the data or a \code{rollcall} data object from package \code{pscl}.
 #' @param outcome Column name of the outcome in \code{score_data}, default is \code{"outcome"}
@@ -57,15 +75,17 @@
 #' Optional, default is \code{'group_id'}. Should be integer, character or factor.
 #' @param person_cov A one-sided formula that specifies the covariates
 #' in \code{score_data} that will be used to hierarchically model the person/legislator ideal points
-#' @param group_cov A one-sided formula that specifies the covariates
-#' in \code{score_data} that will be used to hierarchically model the person/legislator ideal points 
-#' at the group level. Use this in place of \code{person_cov} if you intend to run a group-level model.
 #' @param item_cov A one-sided formula that specifies the covariates
 #' in \code{score_data} that will be used to hierarchically model the 
 #' item/bill discrimination parameters for the regular model
 #' @param item_cov_miss A one-sided formula that specifies the covariates
 #' in the dataset that will be used to hierarchically model the item/bill discrimination parameters for the
 #' missing data model.
+#' @param remove_cov_int Whether to remove constituent terms from hierarchical covariates that 
+#' interact covariates with IDs like \code{person_id} or \code{item_id}. Set to \code{TRUE} if
+#' including these constituent terms would cause multi-collinearity with other terms in the model
+#' (such as running a group-level model with a group-level interaction or a person-level model
+#' with a person-level interaction).
 #' @param simul_data Optionally, data that has been generated by the \code{\link{id_sim_gen}} function.
 #' @param miss_val The value (numeric or character) that indicate missing data/absences in the data. 
 #' If missing data is coded as \code{NA}, 
@@ -130,9 +150,9 @@ id_make <- function(score_data=NULL,
                     group_id='group_id',
                     simul_data=NULL,
                            person_cov=NULL,
-                    group_cov=NULL,
                   item_cov=NULL,
                            item_cov_miss=NULL,
+                  remove_cov_int=FALSE,
                            miss_val=NA,high_val=NULL,low_val=NULL,middle_val=NULL,
                     unbounded=FALSE,
                            exclude_level=NA,simulation=FALSE) {
@@ -158,6 +178,13 @@ id_make <- function(score_data=NULL,
   } 
   
     # data is already in long form
+  
+    # save original ID names as strings to filter later
+  
+    orig_id <- c(item_id,
+                 group_id,
+                 person_id)
+  
     # test for input and quote
     outcome <- .check_quoted(outcome,quo(outcome))
     item_id <- .check_quoted(item_id,quo(item_id))
@@ -217,82 +244,99 @@ id_make <- function(score_data=NULL,
     
     score_rename <- slice(score_rename,as.numeric(attr(personm,'dimnames')[[1]]))
     
-    if(nrow(score_rename)!=nrow(personm)) stop('Covariate matrix and data matrix not the same size.')
+    if(nrow(score_rename)!=nrow(personm)) stop('Covariate matrix and data matrix not the same size even after removing missing data.')
     
-    person_cov <- bind_cols(as_data_frame(personm),
-                            select(score_rename,!! time_id,
-                                   !!person_id))
+    # need to remove any constituent terms for IDs from the model matrix to avoid collinearity with 
+    # model parameters
     
-    # convert to long form for function
+    check_ids <- sapply(orig_id, function(c) {
+      check_all <- grepl(x=colnames(personm),
+                         pattern=c)
+      check_all_colon <- grepl(x=colnames(personm),
+                               pattern=":")
+      check_all & !check_all_colon
+    }) %>% apply(1,any)
     
-    person_cov <- gather(person_cov,key = !! quo(labels),value = !!quo(variables),
-                         -!!time_id,-!!person_id) %>% 
-      distinct
-    
-    person_cov <- .create_array(input_matrix=person_cov,
-                                row_var=person_id,
-                                col_var_name=labels,
-                                col_var_value=variables,
-                                third_dim_var=time_id)
-    
-    # reorder to fit stan
-    person_cov <- aperm(person_cov,c(3,1,2))
+    if(remove_cov_int) {
+      personm <- personm[,!check_ids] 
+    }
+
+    score_rename <- bind_cols(score_rename,
+                              as_data_frame(personm))
+    person_cov <- dimnames(personm)[[2]]
   } else {
-    person_cov <- .create_array(matrix(rep(0,num_person),nrow=num_person,ncol=1),arr_dim=max_t)
-    person_cov <- aperm(person_cov,c(3,1,2))
-  }
-  
-  # separate parameters for group-level covariates
-  
-  if(!is.null(group_cov)) {
-    
-    groupm <- model.matrix(group_cov,data=score_data)
-    
-    # need to check for missing data and remove any missing from IDs
-    
-    score_rename <- slice(score_rename,as.numeric(attr(groupm,'dimnames')[[1]]))
-    
-    group_cov <- bind_cols(as_data_frame(groupm),
-                            select(score_rename,!! time_id,
-                                   !!group_id))
-    
-    # convert to long form for function
-    
-    group_cov <- gather(group_cov,key = !! quo(labels),value = !!quo(variables),
-                         -!!time_id,-!!group_id) %>% 
-      distinct
-    
-    group_cov <- .create_array(input_matrix=group_cov,
-                                row_var=group_id,
-                                col_var_name=labels,
-                                col_var_value=variables,
-                                third_dim_var=time_id)
-    
-    # reorder to fit stan
-    group_cov <- aperm(group_cov,c(3,1,2))
-  } else {
-    group_cov <- .create_array(matrix(rep(0,num_group),nrow=num_group,ncol=1),arr_dim=max_t)
-    group_cov <- aperm(group_cov,c(3,1,2))
+    # make a dummy column if no covariate data
+    score_rename$personcov0 <- 0
+    person_cov <- 'personcov0'
   }
   
   if(!is.null(item_cov)) {
     
-    itemm <- select(score_data, !!! all.vars(item_cov)) %>% 
-      mutate(item_id=pull(score_rename,!!item_id)) %>% 
-      distinct(!!item_id,.keep_all = T)
+    itemm <- model.matrix(item_cov,data=score_data)
     
-    item_cov <- model.matrix(item_cov,data=itemm)
-  } else {
-    item_cov <- matrix(rep(0,num_item),nrow=num_item,ncol=1) 
+    # need to check for missing data and remove any missing from IDs
+    
+    score_rename <- slice(score_rename,as.numeric(attr(itemm,'dimnames')[[1]]))
+    
+    if(nrow(score_rename)!=nrow(itemm)) stop('Covariate matrix and data matrix not the same size even after removing missing data.')
+    
+    # need to remove any constituent terms for IDs from the model matrix to avoid collinearity with 
+    # model parameters
+    
+    check_ids <- sapply(orig_id, function(c) {
+      check_all <- grepl(x=colnames(itemm),
+                         pattern=c)
+      check_all_colon <- grepl(x=colnames(itemm),
+                               pattern=":")
+      check_all & !check_all_colon
+    }) %>% apply(1,any)
+    
+    if(remove_cov_int) {
+      itemm <- itemm[,!check_ids]
     }
-  if(!is.null(item_cov_miss)) {
-    itemm <- select(score_data, !!! all.vars(item_cov)) %>% 
-      mutate(item_id=pull(score_rename,!!item_id)) %>% 
-      distinct(!!item_id,.keep_all = T)
     
-    item_cov_miss <- model.matrix(item_cov_miss,data=itemm)
+    
+    score_rename <- bind_cols(score_rename,
+                              as_data_frame(itemm))
+    item_cov <- dimnames(itemm)[[2]]
   } else {
-    item_cov_miss <- matrix(rep(0,num_item),nrow=num_item,ncol=1) 
+    # make a dummy column if no covariate data
+    score_rename$itemcov0 <- 0
+    item_cov <- 'itemcov0'
+  }
+  
+  if(!is.null(item_cov_miss)) {
+    
+    itemmissm <- model.matrix(item_cov_miss,data=score_data)
+    
+    # need to check for missing data and remove any missing from IDs
+    
+    score_rename <- slice(score_rename,as.numeric(attr(itemmissm,'dimnames')[[1]]))
+    
+    if(nrow(score_rename)!=nrow(itemmissm)) stop('Covariate matrix and data matrix not the same size even after removing missing data.')
+    
+    # need to remove any constituent terms for IDs from the model matrix to avoid collinearity with 
+    # model parameters
+    
+    check_ids <- sapply(orig_id, function(c) {
+      check_all <- grepl(x=colnames(itemmissm),
+                         pattern=c)
+      check_all_colon <- grepl(x=colnames(itemmissm),
+                               pattern=":")
+      check_all & !check_all_colon
+    }) %>% apply(1,any)
+    
+    if(remove_cov_int) {
+      itemmissm <- itemmissm[,!check_ids]
+    }
+
+    score_rename <- bind_cols(score_rename,
+                              as_data_frame(itemmissm))
+    item_cov_miss <- dimnames(itemmissm)[[2]]
+  } else {
+    # make a dummy column if no covariate data
+    score_rename$itemcovmiss0 <- 0
+    item_cov_miss <- 'itemcovmiss0'
   }
   
   # recode score/outcome
@@ -335,7 +379,6 @@ id_make <- function(score_data=NULL,
   outobj <- new('idealdata',
       score_matrix=score_rename,
       person_cov=person_cov,
-      group_cov=group_cov,
       item_cov=item_cov,
       item_cov_miss=item_cov_miss,
       miss_val=miss_val)
@@ -423,13 +466,9 @@ id_make <- function(score_data=NULL,
 #' 
 #' @section Covariates
 #' 
-#' Covariates can be fit on either group-level or person-level ideal point parameters as well as
-#' item discrimination parameters for either the inflated (missing) or non-inflated (observed) 
-#' models. These covariates must be columns that were included with the data fed to the 
-#' \code{\link{id_make}} function. The covariate relationships are specified as 
-#' one-sided formulas, i.e. \code{~cov1 + cov2 + cov1*cov2}. To interact covariates with the 
-#' person-level ideal points you can use \code{~cov1 + person_id + cov1*person_id} and for
-#' group-level ideal poins you can use \code{~cov1 + group_id + cov1*group_id}.
+#' Covariates are included in the model if they were specified as options to the 
+#' \code{\link{id_make}} function. The covariate plots can be accessed with 
+#' \code{\link{id_plot_cov}} on a fitted \code{idealstan} model object.
 #' 
 #' @section Identification:
 #' Identifying IRT models is challenging, and ideal point models are still more challenging 
@@ -481,7 +520,7 @@ id_make <- function(score_data=NULL,
 #' @param use_vb Whether or not to use Stan's variational Bayesian inference engine instead of full Bayesian inference. Pros: it's much faster.
 #' Cons: it's not quite as accurate. See \code{\link[rstan]{vb}} for more info.
 #' @param tol_rel_obj If \code{use_vb} is \code{TRUE}, this parameter sets the stopping rule for the \code{vb} algorithm. 
-#' It's default is 0.0005. A stricter threshold will require the sampler to run longer but may yield a
+#' It's default is 0.001. A stricter threshold will require the sampler to run longer but may yield a
 #' better result in a difficult model with highly correlated parameters. Lowering the threshold should work fine for simpler
 #' models.
 #' @param warmup The number of iterations to use to calibrate Stan's sampler on a given model. Shouldn't be less than 100. 
@@ -583,7 +622,7 @@ id_make <- function(score_data=NULL,
 #'                        decreasing=TRUE,
 #'                        index=TRUE)$ix[1],
 #'                        restrict_ind_low = 
-#'                        sort(bin_irt_2pl_abs_sim@simul_data$true_person
+#'                        sort(bin_irt_2pl_abs_sim@simul_data$true_person,
 #'                        decreasing=FALSE,
 #'                        index=TRUE)$ix[1],
 #'                        fixtype='vb_partial',
@@ -620,7 +659,7 @@ id_make <- function(score_data=NULL,
 #' low_val='No',
 #' miss_val='Absent')
 #' 
-#' sen_est <- id_estimate(senate_data,
+#' sen_est <- id_estimate(to_idealstan,
 #' model_type = 2,
 #' use_vb = TRUE,
 #' fixtype='vb_partial',
@@ -640,7 +679,7 @@ id_make <- function(score_data=NULL,
 #'    \item Betancourt, Michael. "Robust Gaussian Processes in Stan". (October 2017). Case Study.
 #' }
 #' @importFrom stats dnorm dpois model.matrix qlogis relevel rpois update
-#' @importForm utils person
+#' @importFrom utils person
 #' @export
 id_estimate <- function(idealdata=NULL,model_type=2,
                         inflate_zero=FALSE,
@@ -657,8 +696,8 @@ id_estimate <- function(idealdata=NULL,model_type=2,
                         prior_fit=NULL,
                         warmup=floor(niters/2),ncores=4,
                         use_groups=FALSE,
-                           discrim_reg_sd=1,
-                           discrim_miss_sd=1,
+                           discrim_reg_sd=2,
+                           discrim_miss_sd=2,
                            person_sd=1,
                         time_sd=.1,
                         sample_stationary=FALSE,
@@ -671,7 +710,7 @@ id_estimate <- function(idealdata=NULL,model_type=2,
                         restrict_mean_val=NULL,
                         restrict_mean_ind=NULL,
                         restrict_var_high=0.1,
-                        tol_rel_obj=.0001,
+                        tol_rel_obj=.001,
                         gp_sd_par=.025,
                         gp_num_diff=c(3,0.01),
                         gp_m_sd_par=c(0.3,10),
@@ -699,8 +738,8 @@ id_estimate <- function(idealdata=NULL,model_type=2,
   if(vary_ideal_pts=='none') {
     idealdata@score_matrix$time_id <- 1
     # make sure that the covariate arrays are only one time point
-    idealdata@person_cov <- idealdata@person_cov[1,,,drop=F]
-    idealdata@group_cov <- idealdata@group_cov[1,,,drop=F]
+    #idealdata@person_cov <- idealdata@person_cov[1,,,drop=F]
+    #idealdata@group_cov <- idealdata@group_cov[1,,,drop=F]
   } 
   
   vary_ideal_pts <- switch(vary_ideal_pts,
@@ -713,7 +752,8 @@ id_estimate <- function(idealdata=NULL,model_type=2,
   
   if(vary_ideal_pts==4) {
     # convert multiplicity factor to total length of the data
-    gp_num_diff[1] <- length(unique(idealdata@score_matrix$time_id))*gp_num_diff[1]
+    # use real time points instead of just counting number of points
+    gp_num_diff[1] <- mean(as.numeric(idealdata@score_matrix$time_id))*gp_num_diff[1]
   }
     
   # use either row numbers for person/legislator IDs or use group IDs (static or time-varying)
@@ -721,15 +761,9 @@ id_estimate <- function(idealdata=NULL,model_type=2,
   if(use_groups==T) {
     legispoints <- as.numeric(idealdata@score_matrix$group_id)
     num_legis <- max(legispoints)
-    # this handles the situation in which the data is fake and only 
-    # groups are used as parameters
-    legis_pred <- idealdata@group_cov
-    lx <- dim(idealdata@group_cov)[3]
   } else {
     legispoints <- as.numeric(idealdata@score_matrix$person_id)
     num_legis <- max(legispoints)
-    legis_pred <- idealdata@person_cov
-    lx <- dim(idealdata@person_cov)[3]
   }
 
   billpoints <- as.numeric(idealdata@score_matrix$item_id)
@@ -862,13 +896,15 @@ id_estimate <- function(idealdata=NULL,model_type=2,
                     bb=billpoints,
                     num_fix_high=as.integer(1),
                     num_fix_low=as.integer(1),
-                    LX=lx,
-                    SRX=ncol(idealdata@item_cov),
-                    SAX=ncol(idealdata@item_cov_miss),
-                    legis_pred=legis_pred,
-                    group_pred=idealdata@group_cov,
-                    srx_pred=idealdata@item_cov,
-                    sax_pred=idealdata@item_cov_miss,
+                    LX=length(idealdata@person_cov),
+                    SRX=length(idealdata@item_cov),
+                    SAX=length(idealdata@item_cov_miss),
+                    legis_pred=as.matrix(select(idealdata@score_matrix,
+                                                idealdata@person_cov))[remove_nas,,drop=F],
+                    srx_pred=as.matrix(select(idealdata@score_matrix,
+                                              idealdata@item_cov))[remove_nas,,drop=F],
+                    sax_pred=as.matrix(select(idealdata@score_matrix,
+                                              idealdata@item_cov_miss))[remove_nas,,drop=F],
                     time=timepoints,
                     model_type=model_type,
                     discrim_reg_sd=discrim_reg_sd,
@@ -900,6 +936,7 @@ id_estimate <- function(idealdata=NULL,model_type=2,
                         nfix=nfix,restrict_ind_high=restrict_ind_high,
                         restrict_ind_low=restrict_ind_low,
                         ncores=ncores,
+                        tol_rel_obj=tol_rel_obj,
                         use_groups=use_groups,
                         prior_fit=prior_fit)
 
@@ -915,11 +952,9 @@ id_estimate <- function(idealdata=NULL,model_type=2,
   if(use_groups==T) {
     legispoints <- as.numeric(idealdata@score_matrix$group_id)
     num_legis <- max(legispoints)
-    lx <- dim(idealdata@group_cov)[3]
   } else {
     legispoints <- as.numeric(idealdata@score_matrix$person_id)
     num_legis <- max(legispoints)
-    lx <- dim(idealdata@person_cov)[3]
   }
   
   billpoints <- as.numeric(idealdata@score_matrix$item_id)
@@ -974,13 +1009,15 @@ id_estimate <- function(idealdata=NULL,model_type=2,
                     bb=billpoints,
                     num_fix_high=as.integer(1),
                     num_fix_low=as.integer(1),
-                    LX=lx,
-                    SRX=ncol(idealdata@item_cov),
-                    SAX=ncol(idealdata@item_cov_miss),
-                    legis_pred=legis_pred,
-                    group_pred=idealdata@group_cov,
-                    srx_pred=idealdata@item_cov,
-                    sax_pred=idealdata@item_cov_miss,
+                    LX=length(idealdata@person_cov),
+                    SRX=length(idealdata@item_cov),
+                    SAX=length(idealdata@item_cov_miss),
+                    legis_pred=as.matrix(select(idealdata@score_matrix,
+                                                idealdata@person_cov))[remove_nas,,drop=F],
+                    srx_pred=as.matrix(select(idealdata@score_matrix,
+                                              idealdata@item_cov))[remove_nas,,drop=F],
+                    sax_pred=as.matrix(select(idealdata@score_matrix,
+                                              idealdata@item_cov_miss))[remove_nas,,drop=F],
                     time=timepoints,
                     time_proc=vary_ideal_pts,
                     model_type=model_type,
@@ -1005,7 +1042,8 @@ id_estimate <- function(idealdata=NULL,model_type=2,
                     gp_sd_par=gp_sd_par,
                     num_diff=gp_num_diff,
                     m_sd_par=gp_m_sd_par,
-                    min_length=gp_min_length)
+                    min_length=gp_min_length,
+                    id_refresh=id_refresh)
 
   outobj <- sample_model(object=idealdata,nchains=nchains,niters=niters,warmup=warmup,ncores=ncores,
                          this_data=this_data,use_vb=use_vb,
