@@ -995,48 +995,511 @@ id_plot_rhats <- function(obj) {
     ylab('Parameters') +theme(panel.grid=element_blank())
 }
 
-#' Display Coefficient Plot of Hierarchical Covariates
+#' Marginal Effects Plot for Hierarchical Covariates
 #' 
-#' This function will pull the estimates of the hierarchical covariates (whether at the person or 
-#' item-discrimination level) and then plot them on a vertical coefficient plot. Names of 
-#' the parameters are taken from the levels of the factor if a categorical variable and the
-#' column names otherwise. 
+#' This function will calculate marginal effects, or the first derivative
+#' of the IRT/ideal point model with respect to the hierarchical covariate,
+#' separately for the two poles of the latent variable. These two marginal
+#' effects permit the interpretation of the effect of the covariate on 
+#' with respect to either end of the latent variable.
+#' 
+#' Because the marginal effects are always with respect to a given
+#' outcome/response, the outcome to be predicted must be specified 
+#' in \code{pred_outcome}. If it is not specified, the function
+#' will prompt you to select one of the outcome's values in the data.
+#' 
+#' The ends of the latent variable can be specified via the 
+#' \code{label_low} and \code{label_high} options, which will use those
+#' labels in the ensuing plot.
+#' 
+#' To exclude parameters from the plot, use the \code{filter_cov} option. 
+#' Note that the parameters must be specified using the underlying model 
+#' syntax (however they are labeled in the plot). You can also change
+#' the names of parameters using the \code{new_cov_names} option.
+#' 
+#' Note that the function produces a \code{ggplot2} object, which can 
+#' be further modified with \code{ggplot2} functions.
 #' 
 #' @param object A fitted \code{idealstan} object
+#' @param calc_varying Whether to marginalize covariate effects over 
+#' discrimination parameters to calculate a meaningful quantity for the effect of
+#' covariates on the latent scale (see vignette). Defaults to \code{TRUE}
+#' @param label_high What label to use on the plot for the high end of the 
+#' latent scale
+#' @param label_low What label to use on the plot for the low end of the 
+#' latent scale
+#' @param pred_outcome For discrete models with more than 2 categories, 
+#' or binary models with missing data, which outcome to predict. This should 
+#' be a character value that matches what the outcome was coded as in the data
+#' passed to \code{\link{id_make}}.
+#' @param high_quantile The upper limit of the posterior density to use for 
+#' calculating credible intervals
+#' @param low_quantile The lower limit of the posterior density to use for
+#' calculating credible intervals
+#' @param new_cov_names A character vector of length equal to the number
+#' of covariates (plus 1 for the intercept) to change the default labels.
+#' To see the default labels, use the plot function with this option blank.
+#' The character vector should be of th form used by 
 #' @param cov_type Either \code{'person_cov'} for person or group-level hierarchical parameters,
 #' \code{'discrim_reg_cov'} for bill/item discrimination parameters from regular (non-inflated) model, and 
 #' \code{'discrim_infl_cov'} for bill/item discrimination parameters from inflated model.
 #' @param filter_cov A character vector of coefficients from covariate plots to exclude from
 #' plotting (should be the names of coefficients as they appear in the plots)
-#' @param ... Any additional parameters passed on to \code{\link[bayesplot]{mcmc_intervals}}
+#' @param recalc_vals A character value of length three that can be used to create
+#' a new variable that is a sum of two other variables. The first two values of the
+#' character vector are the names of these parameters, while the third value is the name
+#' of the new combined variable. Note that if the parameters are renamed, the new names
+#' should be used in this option.
 #' @return A \code{ggplot2} plot that can be further customized with \code{ggplot2} functions if need be.
+#' @import svDialogs
 #' @export
 id_plot_cov <- function(object,
+                        calc_varying=T,
+                        label_high="Liberal",
+                        label_low="Conservative",
                         cov_type='person_cov',
+                        pred_outcome = NULL,
+                        high_quantile=0.95,
+                        low_quantile=0.05,
                         filter_cov=NULL,
-                        ...) {
+                        new_cov_names=NULL,
+                        recalc_vals=NULL) {
+  
+  # determine which outcome to predict
+  
+  if(is.null(pred_outcome)) {
+    if(object@model_type %in% c(1,2,3,4,5,6,13,14)) {
+      # ask user for predicted outcome
+      pred_outcome <- svDialogs::dlg_list(levels(object@score_data@score_matrix$outcome),
+                                          title="Select which level of the outcome to predict using covariates.")$res
+    } else if(object@model_type %in% c(7,8)) {
+      pred_outcome <- "Mean Count"
+    } else if(object@model_type %in% c(9,10,11,12)) {
+      pred_outcome <- "Mean"
+    }
+  }
+  
+  # adjust labels to match predicted outcome
+  
+  if(object@model_type %in% c(1,2,3,4,5,6,13,14)) {
+    pred_outcome_high <- paste0("Pr(",pred_outcome,"|",label_high,")")
+    pred_outcome_low <- paste0("Pr(",pred_outcome,"|",label_low,")")
+    xlabel <- "Marginal Change in Probability"
+  } else if(object@model_type %in% c(7,8)) {
+    pred_outcome_high <- paste0("Mean Count|",label_high)
+    pred_outcome_low <- paste0("Mean Count|",label_low)
+    xlabel <- "Marginal Change in Mean Count"
+  } else {
+    pred_outcome_high <- paste0("Mean|",label_high)
+    pred_outcome_low <- paste0("Mean|",label_low)
+    xlabel <- "Marginal Change in Mean"
+  }
+  
+  # pull hierarchical covariates
   
   param_name <- switch(cov_type,person_cov='legis_x',
                        discrim_reg_cov='sigma_reg_x',
                        discrim_infl_cov='sigma_abs_x')
   
   to_plot <- as.array(object@stan_samples,
-                   pars=param_name)
-  
-  if(object@use_groups && cov_type=='person_cov') {
-    cov_type <- 'group_cov'
-  }
+                      pars=param_name)
   
   # reset names of parameters
-  new_names <- switch(cov_type,person_cov=attributes(object@score_data@person_cov)$dimnames$colnames,
-                      group_cov=attributes(object@score_data@group_cov)$dimnames$colnames,
-                      discrim_reg=attributes(object@score_data@item_cov)$dimnames$colnames,
-                      discrim_abs=attributes(object@score_data@item_cov_miss)$dimnames$colnames)
-
-  attributes(to_plot)$dimnames$parameters <- new_names
-  if(!is.null(filter_cov)) {
-    to_plot <- to_plot[,,new_names %in% filter_cov]
+  new_names <- switch(cov_type,person_cov=object@score_data@person_cov,
+                      discrim_reg=object@score_data@item_cov,
+                      discrim_abs=object@score_data@item_cov_miss)
+  
+  # recode these names if user supplies option
+  
+  if(!is.null(new_cov_names)) {
+    new_names <- recode(new_names,!!!new_cov_names)
   }
+  
+  attributes(to_plot)$dimnames$parameters <- new_names
+  
+  # remove unwanted coefficients
+  
+  if(!is.null(filter_cov)) {
+    to_plot <- to_plot[,,!(new_names %in% filter_cov),drop=F]
+    new_names <- new_names[!(new_names %in% filter_cov)]
+    new_cov_names <- new_cov_names[!(new_cov_names %in% filter_cov)]
+  }
+  
+  # set up values to re-calculate
+  
+  if(!is.null(recalc_vals)) {
+    if(length(recalc_vals)!=3) {
+      stop("Option recalc_vals can only be a character vector of length 3 indicating which two variables to add together and their name.")
+    }
+    val1 <- which(attributes(to_plot)$dimnames$parameters==recalc_vals[1])
+    val2 <- which(attributes(to_plot)$dimnames$parameters==recalc_vals[2])
+    if(is.null(val1) || is.null(val2)) {
+      stop("The parameter names you passed to re-calculate did not match existing parameters. Please be sure to use recoded parameter names not original parameter names.")
+    }
+  }
+  
+  if(calc_varying) {
+    # get all sigmas
+    sigma_all <- rstan::extract(object@stan_samples,"sigma_reg_free")
+    
+    # iterate over posterior draws and calculate effect conditional on pos/neg discrimination
+    # for all params in to_plot
+    
+    neg_eff <- lapply(1:nrow(sigma_all[[1]]), function(i) {
+      this_discrim <- sigma_all[[1]][i,]
+      
+      neg_discrim <- this_discrim[this_discrim<0]
+      
+      #calculate marginal changes in probability
+      to_plot_neg <- apply(to_plot,3,function(c) {
+        mean(plogis(c[i]*neg_discrim)-0.5)
+      })
+      tibble(estimate=to_plot_neg,
+             parameter=names(to_plot_neg)) %>% 
+        mutate(Type=pred_outcome_low)
+    }) %>% bind_rows
+    
+    pos_eff <- lapply(1:nrow(sigma_all[[1]]), function(i) {
+      this_discrim <- sigma_all[[1]][i,]
+      pos_discrim <- this_discrim[this_discrim>0]
+      
+      
+      #calculate marginal changes in probability
+      to_plot_neg <- apply(to_plot,3,function(c) {
+        mean(plogis(c[i]*pos_discrim)-0.5)
+      })
+      tibble(estimate=to_plot_neg,
+             parameter=names(to_plot_neg)) %>% 
+        mutate(Type=pred_outcome_high)
+    }) %>% bind_rows
+    
+    if(!is.null(recalc_vals)) {
+      # do the same for re-calculated values
+      
+      neg_eff_recalc <- lapply(1:nrow(sigma_all[[1]]), function(i) {
+        this_discrim <- sigma_all[[1]][i,]
+        
+        neg_discrim <- this_discrim[this_discrim<0]
+        
+        #calculate marginal changes in probability
+        to_plot_neg <- mean(plogis((to_plot[i,,val1]+to_plot[i,,val2])*neg_discrim)-0.5)
+        
+        tibble(estimate=to_plot_neg,
+               parameter=recalc_vals[3]) %>% 
+          mutate(Type=pred_outcome_low)
+      }) %>% bind_rows
+      
+      pos_eff_recalc <- lapply(1:nrow(sigma_all[[1]]), function(i) {
+        this_discrim <- sigma_all[[1]][i,]
+        pos_discrim <- this_discrim[this_discrim>0]
+        
+        
+        #calculate marginal changes in probability
+        to_plot_neg <- mean(plogis((to_plot[i,,val1]+to_plot[i,,val2])*pos_discrim)-0.5)
+        
+        tibble(estimate=to_plot_neg,
+               parameter=recalc_vals[3]) %>% 
+          mutate(Type=pred_outcome_high)
+      }) %>% bind_rows
+    }
+    
+    if(!is.null(recalc_vals)) {
+      to_plot <- bind_rows(neg_eff,
+                           pos_eff,
+                           neg_eff_recalc,
+                           pos_eff_recalc)
+    } else {
+      to_plot <- bind_rows(neg_eff,
+                           pos_eff)
+    }
+    
+    
+    
+    sum_func <- function(this_data,high=high_quantile,
+                         low=low_quantile) {
+      tibble(y=median(this_data),
+             ymin=quantile(this_data,low_quantile),
+             ymax=quantile(this_data,high_quantile))
+    }
+    
+    # if new levels exist, reorder
+    
+    if(!is.null(new_cov_names)) {
+      if(is.null(recalc_vals)) {
+        to_plot$parameter <- fct_relevel(factor(to_plot$parameter),rev(new_cov_names))
+      } else {
+        to_plot$parameter <- fct_relevel(factor(to_plot$parameter),c(rev(new_cov_names),recalc_vals[3]))
+      }
+      
+    }
+    
+    outplot <- to_plot %>% 
+      ggplot(aes(x=parameter,y=estimate)) +
+      stat_summary(fun.data=sum_func) +
+      coord_flip() +
+      geom_hline(yintercept=0,linetype=2) +
+      facet_wrap(~Type) +
+      theme(panel.grid=element_blank(),
+            panel.background = element_blank(),
+            strip.background = element_blank(),
+            strip.text = element_text(face="bold"),
+            axis.ticks.y=element_blank()) +
+      xlab("") + 
+      ylab(xlabel)
+    
+    if(object@model_type %in% c(1,2,3,4,5,6,13,14)) {
+      outplot <- outplot + scale_y_continuous(labels=scales::percent)
+    }
+    
+    return(outplot)
+    
+  } else {
+    
+    mcmc_intervals(to_plot) + xlab("Ideal Point Score")
+  }
+  
+}
 
-  mcmc_intervals(to_plot,...) + xlab('Ideal Point Score')
-}
\ No newline at end of file
+#' Generate Impulse Response Functions for Covariates
+#' 
+#' This function will generate an impulse response function (IRF)
+#' for a given covariate. The IRF shows the marginal impact of a 1-unit
+#' change in the covariate on a person's ideal point over time. To use 
+#' this function, the \code{vary_ideal_pts} option in 
+#' \code{\link{id_estimate}} must have received the \code{'AR1'} option
+#' as IRFs are only available for the AR(1) auto-regressive model.
+#' 
+#' @param object A fitted \code{idealstan} object
+#' @param cov_name The name of the covariate to plot. Leave blank to select 
+#' from a list of available covariates
+#' @param calc_varying if \code{TRUE}, will calculate marginal effects of the
+#' covariates on each end of the latent scale (see vignette for more information)
+#' @param label_high The character label for the upper end of the latent scale
+#' @param label_low The character label for the lower end of the latent scale
+#' @param pred_outcome For discrete models with more than 2 categories, 
+#' or binary models with missing data, which outcome to predict. This should 
+#' be a character value that matches what the outcome was coded as in the data
+#' passed to \code{\link{id_make}}.
+#' @param high_quantile The upper limit of the posterior density to use for 
+#' calculating credible intervals
+#' @param low_quantile The lower limit of the posterior density to use for
+#' calculating credible intervals
+#' @param recalc_vals Whether to combine two variables into one through addition before
+#' computing IRFs. If \code{TRUE}, two names of parameters should be passed to 
+#' \code{cov_name} or selected from the dialog list
+#' @param include A list of character names of person or group IDs for which to
+#' calculate IRFs
+#' @param time_calc The maximum number of time points over which to calculate the
+#' IRF
+#' @param time_label Character string specifying the type of time points (default is just
+#' \code{"Time Points"})
+#' @param line_type The line type of the IRF line (see \code{ggplot2} documentation)
+#' @param line_width The line width of the IRF line (see \code{ggplot2} documentation)
+#' @param line_alpha The line alpha (transparency) of the IRF line (see \code{ggplot2} documentation)
+#' @param line_color The color of the IRF line (see \code{ggplot2} documentation)
+#' @param ci_color The color of the IRF credible interval (see \code{ggplot2} documentation)
+#' @param ci_alpha The alpha of the IRF credible interval (see \code{ggplot2} documentation)
+#' @param use_ci Whether or not to plot a credible interval around the lines
+#' @return a \code{ggplot2} object that can be further customized if necessary
+#' @export
+#' @import scales
+id_plot_irf <- function(object,
+                        cov_name=NULL,
+                        label_high="Liberal",
+                        label_low="Conservative",
+                        pred_outcome=NULL,
+                        recalc_vals=F,
+                        include=NULL,
+                        time_calc=10,
+                        time_label="Time Points",
+                        line_type=2,
+                        line_width=1,
+                        line_alpha=1,
+                        line_color="red",
+                        ci_color='black',
+                        ci_alpha=0.5,
+                        use_ci=TRUE,
+                        high_quantile=0.95,
+                        low_quantile=0.05,
+                        calc_varying=T) {
+  
+  # figure out which covariate to iterate over
+  
+  # determine which outcome to predict
+  
+  if(is.null(pred_outcome)) {
+    if(object@model_type %in% c(1,2,3,4,5,6,13,14)) {
+      # ask user for predicted outcome
+      pred_outcome <- svDialogs::dlg_list(levels(object@score_data@score_matrix$outcome),
+                                          title="Select which level of the outcome to predict using covariates.")$res
+    } else if(object@model_type %in% c(7,8)) {
+      pred_outcome <- "Mean Count"
+    } else if(object@model_type %in% c(9,10,11,12)) {
+      pred_outcome <- "Mean"
+    }
+  }
+  
+  # adjust labels to match predicted outcome
+  
+  if(object@model_type %in% c(1,2,3,4,5,6,13,14)) {
+    pred_outcome_high <- paste0("Pr(",pred_outcome,"|",label_high,")")
+    pred_outcome_low <- paste0("Pr(",pred_outcome,"|",label_low,")")
+    xlabel <- "Marginal Change in Probability"
+  } else if(object@model_type %in% c(7,8)) {
+    pred_outcome_high <- paste0("Mean Count|",label_high)
+    pred_outcome_low <- paste0("Mean Count|",label_low)
+    xlabel <- "Marginal Change in Mean Count"
+  } else {
+    pred_outcome_high <- paste0("Mean|",label_high)
+    pred_outcome_low <- paste0("Mean|",label_low)
+    xlabel <- "Marginal Change in Mean"
+  }
+  
+  # pull hierarchical covariates
+  
+  param_name <- "legis_x"
+  
+  # figure out which prameter to use
+  
+  # reset names of parameters
+  new_names <- object@score_data@person_cov
+  
+  if(is.null(cov_name)) {
+    cov_name <- svDialogs::dlg_list(new_names,multiple=T,
+                                    title="Select at least one variable to compute IRFs. If you want to combine two variables, select two variables (but not more than two).")$res
+  }
+  
+  to_plot <- as.array(object@stan_samples,
+                      pars=param_name)
+  
+  attributes(to_plot)$dimnames$parameters <- new_names
+  
+  to_plot <- to_plot[,,(new_names %in% cov_name),drop=F]
+  
+  # now need to loop over persons/item IDs to generate IRFs
+  
+  if(object@use_groups) {
+    all_ids <- unique(object@score_data@score_matrix$group_id)
+  } else {
+    all_ids <- unique(object@score_data@score_matrix$person_id)
+  }
+  
+  ar1 <- rstan::extract(object@stan_samples,"L_AR1")[[1]]
+  
+  # keep some if user specifies
+  if(!is.null(include)) {
+    ar1 <- ar1[,all_ids %in% include]
+    all_ids <- all_ids[all_ids %in% include]
+  }
+  
+  # calc IRF first then convert to marginal changes if wanted
+  
+  # get cov values
+  
+  cov1 <- to_plot[,,1]
+  
+  if(recalc_vals) {
+    cov1 <- cov1 + to_plot[,,2]
+  }
+  
+  # iterate over all the possible AR1 parameters
+  
+  all_irfs <- apply(ar1,2, function(arp) {
+    this_irf <- .irf(total_t=time_calc,
+                     adj_in=arp,
+                     shock=cov1)  
+    return(this_irf)
+  })
+  
+  names(all_irfs) <- all_ids
+  
+  all_irfs <- bind_rows(all_irfs,.id="Person") 
+  
+  
+  
+  if(calc_varying) {
+    
+    # pull sigmas if we want to calculate marginal changes
+    
+    sigma_all <- rstan::extract(object@stan_samples,"sigma_reg_free")[[1]]
+    
+    # iterate over persons and time points
+    
+    
+    all_eff <- lapply(1:nrow(sigma_all), function(i) {
+      this_discrim <- sigma_all[i,]
+      pos_discrim <- this_discrim[this_discrim>0]
+      neg_discrim <- this_discrim[this_discrim<0]
+      
+      this_data <- filter(all_irfs,iter==i) %>% 
+        group_by(Person,time) %>% 
+        mutate(marg_neg=mean(plogis(y_shock*neg_discrim)-0.5),
+               marg_pos=mean(plogis(y_shock*pos_discrim)-0.5))
+    }) %>% bind_rows
+    
+    outplot <- all_eff %>% gather(key=Type,value=estimate,marg_neg,marg_pos) %>% 
+      group_by(Person,time,Type) %>% 
+      summarize(mean_est=mean(estimate),
+                high_est=quantile(estimate,high_quantile),
+                low_est=quantile(estimate,low_quantile)) %>% 
+      ungroup %>% 
+      mutate(time=factor(time),
+             Type=recode(Type,marg_neg=pred_outcome_low,
+                         marg_pos=pred_outcome_high)) %>% 
+      ggplot(aes(y=mean_est,x=time,group=Person))
+    
+    if(use_ci) {
+      outplot <- outplot + geom_ribbon(aes(ymin=low_est,
+                                           ymax=high_est),
+                                       alpha=ci_alpha,
+                                       fill=ci_color)
+    }
+    
+    outplot + geom_line(linetype=line_type,colour=line_color,size=line_width,
+                        alpha=line_alpha) +
+      geom_hline(yintercept=0,linetype=3) +
+      facet_wrap(~Type) +
+      theme(panel.grid=element_blank(),
+            panel.background = element_blank(),
+            strip.background = element_blank(),
+            strip.text = element_text(face="bold"),
+            axis.ticks.y=element_blank()) +
+      scale_y_continuous(labels = scales::percent) +
+      xlab(time_label) + 
+      ylab(xlabel)
+    
+    
+  } else {
+    # don't need as many fancy options if we aren't calculating marginal changes
+    
+    outplot <- all_irfs %>% group_by(Person,time) %>% 
+      summarize(mean_est=mean(y_shock),
+                high_est=quantile(y_shock,high_quantile),
+                low_est=quantile(y_shock,low_quantile)) %>% 
+      ungroup %>% 
+      mutate(time=factor(time)) %>% 
+      ggplot(aes(y=mean_est,x=time,group=Person))
+    
+    if(use_ci) {
+      outplot <- outplot + geom_ribbon(aes(ymin=low_est,
+                                           ymax=high_est),
+                                       alpha=ci_alpha,
+                                       fill=ci_color)
+    }
+    
+    outplot + geom_line(linetype=line_type,colour=line_color,size=line_width,
+                        alpha=line_alpha) +
+      geom_hline(yintercept=0,linetype=3) +
+      theme(panel.grid=element_blank(),
+            panel.background = element_blank(),
+            strip.background = element_blank(),
+            strip.text = element_text(face="bold"),
+            axis.ticks.y=element_blank()) +
+      xlab(time_label) + 
+      ylab("Ideal Point Scale")
+  }
+  
+  
+  
+  
+}
